import josx.platform.rcx.Motor;
import josx.platform.rcx.Sensor;
import josx.platform.rcx.TextLCD;

/**
 * LineFollower.java Program for RCX that simply follows a black line
 * on a white board.  Robot is best started in CENTER_ONLY, a state
 * where the center sensor is the only one on the black line. However,
 * it may be able to handle starting in the LEFT_CENTER or
 * RIGHT_CENTER positions. This is untested. <br/><br/>
 *
 * <B>Improvements in Version 2.0</b>
 *
 * The only two things with any changes are the LEFT_CENTER and
 * RIGHT_CENTER clauses. Since these clauses are very similar to
 * eachother, only changes to one will be described. Together, these
 * changes allow the robot to handle turns of as little as twenty-five
 * degrees. Turns smaller than this are thought to be unrealistic.
 *
 * When the robot enters the LEFT_CENTER clause, the robot will first
 * go to sleep for some period of time. This is actually a very poor
 * way of handling the problem, since the sleep method is highly
 * dependent on battary power. We suggest an improvement to this part
 * of the problem. Please see the full web documentation for further
 * suggestions on how to solve this problem.
 *
 * After entering sleep, the robot turns up the power for hte wheels
 * on the outside edge of the turn. This allows the robot to bring the
 * sensors back into the line in such a manner that it will trigger
 * just center only. With the motors at the same power, when turning
 * onto an acute angle turn, the state LEFT_CENTER will trigger again,
 * and the robot will do bad things.
 *
 * At the start of the turn, the robot sleeps some more. We found this
 * to be helpful in keeping the robot off the pointy part of the line
 * on a turn. See the web documentation for more information on
 * this. Finally, the motor powers are returned to the original
 * values.<br/><br/>
 *
 * <B>Improvements in Version 3.0</B>
 * 
 * Robot functions from any position on a line - LEFT, LEFT_CENTER, etc.
 * Rather than only functioning from CENTER_ONLY. We do still assume that
 * the robot starts on a line
 *
 * The most remarkable modifications to the code are the integration in this
 * line follower code of the following elements: (1) battery-voltage 
 * independent actions that replace previous attempts to use sleep threads
 * to turn and letting while loops control the execution of the actions
 * to get to a better state. (2) Improvement over challenge 2 & 3, where
 * the robot interacts with intersections and acute angles very robustly.
 * (3) The robot has the capacity of orienting itself when it finds the
 * end of the line, turning around and going back to where it came from.
 *
 * @author CS371 2006
 * @author <a href="mailto:alleke01@gettysburg.edu">Ken Allen</a>
 * @author <a href="mailto:martle02@gettysburg.edu">Leo Marte</a>
 * @author <a href="mailto:mallgi01@gettysburg.edu">Gitendra Malla</a>
 * @author <a href="mailto:albrmi01@gettysburg.edu">Mike Albretch</a>
 * @author CS371 2004
 * @author CS371 2002
 * @version 3.0
 * @since 1.0
 */
public class LineFollower extends JavaRCX {
  
    /**
     * Runs forever taking appropriate motor actions for each state it
     * is in.
     *
     * @param args a String[] value, command line arguments, none in
     * this case
     */
    public static void main(String[] args){
	initialize();
	while (true) {
	    if(state == NONE){
		//this used to call the lost() method which turned out to be redundant
		//we moved the functionality of lost here because basically what lost() did was
		//turn until it was no longer in NONE
		TextLCD.print("---");
		//turn clockwise until a better state is found
		//it just spins in place if there is no line around
		//this could be better but intersection handling depends on this behaviour
		Motor.A.forward();
		Motor.C.forward();
		while( state == NONE );
	    }else if(state == LEFT_ONLY){
		TextLCD.print("+--");
		//in the previous code it did the same while loop
		//but repeatedly called Motor.backward in it
		//this seemed terribly inefficient so we moved the motor commands out of the loop

		//turn left until centered on the line
		Motor.C.backward();
		Motor.A.backward();
		while(state != CENTER_ONLY );
	    }
	    else if(state == CENTER_ONLY){
		//this state got flooded because there was no while loop
		//therefore it was calling this state too many times
		//now, it only gets out when you get out of the center, instead
		//of calling center a billion times.
		TextLCD.print("-+-");
		//drive forward until no longer centered
		Motor.A.forward();
		Motor.C.backward();
		while(state == CENTER_ONLY );// Todd's addition 
	    }
	    else if(state == RIGHT_ONLY){
		TextLCD.print("--+");
		//this used to be a similar situation to the one described in
		//left_only, same fix

		//turn right until centered on the line
		Motor.A.forward();
		Motor.C.forward();
		while(state != CENTER_ONLY);
	    }
	    else if(state == LEFT_CENTER){
		TextLCD.print("++-");
		//the previous code used several Thread.sleeps to implement specific turns
		//this was not battery-level independent as at lower powers the motors would not drive the
		//robot through the angle they intended

		//turn left until centered on the line
		// ** remaining code from 2.0
		Motor.A.backward();
		Motor.C.backward();

		//the only new code for 3.0
		while( state != CENTER_ONLY );		
	    }
	    else if(state == LEFT_RIGHT){
		TextLCD.print("+-+");

		//turn right until in a different state
		Motor.A.forward();
		Motor.C.backward();
		while( state == LEFT_RIGHT );

		//we no longer use the lost() method as it was redundant
		//lost();
	    }
	    else if(state == CENTER_RIGHT){
		//the previous code did the same thing in this state
		//as in LEFT_CENTER
		TextLCD.print("-++");
			
		//turn right until centered on the line
		Motor.A.forward();
		Motor.C.forward();
		

		while( state != CENTER_ONLY );
		
	    }
	    else if(state == ALL){
		TextLCD.print("+++");
		//drive forward until in a different state
		Motor.A.forward();
		Motor.C.backward();
		while( state == ALL );
		//this method used to call lost() instead of the above line
		//this way results in much better intersection handling
		//at a plus-intersection it will drive forward and find itself on a center
		//and continue
		//at a T-intersection it will drive forward and find itself in a none state,
		//turn right and continue
	    }
	}
    }

    
}// LineFollower

